@@ -1,6 +1,6 @@
 package PDL::NDBin::Action::Count;
 # ABSTRACT: Action for PDL::NDBin that counts elements
-$PDL::NDBin::Action::Count::VERSION = '0.021';
+$PDL::NDBin::Action::Count::VERSION = '0.023';
 
 use strict;
 use warnings;
@@ -24,8 +24,12 @@ sub process
 {
 	my $self = shift;
 	my $iter = shift;
-	$self->{out} = PDL->zeroes( $self->{type}, $self->{N} ) unless defined $self->{out};
-	PDL::NDBin::Actions_PP::_icount_loop( $iter->data, $iter->idx, $self->{out}, $self->{N} );
+	my $out = $self->{out} //= PDL->zeroes( $self->{type}, $self->{N} );
+	my $idx = $iter->idx;
+	$_ = $out->zeroes for grep !defined || (ref && $_->isnull), $idx;
+	my $data = $iter->data;
+	$_ = $idx->zeroes for grep !defined || (ref && $_->isnull), $data;
+	PDL::NDBin::Actions_PP::_icount_loop( $data, $idx, $self->{out}, $self->{N} );
 	# as the plugin processes all bins at once, every variable
 	# needs to be visited only once
 	$iter->var_active( 0 );
@@ -53,7 +57,7 @@ PDL::NDBin::Action::Count - Action for PDL::NDBin that counts elements
 
 =head1 VERSION
 
-version 0.021
+version 0.023
 
 =head1 DESCRIPTION
 
@@ -69,7 +73,7 @@ This class implements an action for PDL::NDBin.
 	);
 
 Construct an instance for this action. Requires the number of bins $N as input.
-Optionally allows the type of the output piddle to be set (defaults to
+Optionally allows the type of the output ndarray to be set (defaults to
 I<indx>).
 
 =head2 process()
@@ -91,7 +95,7 @@ Edward Baudrez <ebaudrez@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Edward Baudrez.
+This software is copyright (c) 2022 by Edward Baudrez.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
