@@ -1,6 +1,6 @@
 package PDL::NDBin::Action::Max;
 # ABSTRACT: Action for PDL::NDBin that computes maximum
-$PDL::NDBin::Action::Max::VERSION = '0.021';
+$PDL::NDBin::Action::Max::VERSION = '0.023';
 
 use strict;
 use warnings;
@@ -28,7 +28,10 @@ sub process
 		my $type = defined $self->{type} ? $self->{type} : $iter->data->type;
 		$self->{out} = PDL->zeroes( $type, $self->{N} )->setbadif( 1 );
 	}
-	PDL::NDBin::Actions_PP::_imax_loop( $iter->data, $iter->idx, $self->{out}, $self->{N} );
+	my $data = $iter->data;
+	my $idx = $iter->idx;
+	$_ = PDL->zeroes( 0 ) for grep $_->isnull, $data, $idx;
+	PDL::NDBin::Actions_PP::_imax_loop( $data, $idx, $self->{out}, $self->{N} );
 	# as the plugin processes all bins at once, every variable
 	# needs to be visited only once
 	$iter->var_active( 0 );
@@ -56,7 +59,7 @@ PDL::NDBin::Action::Max - Action for PDL::NDBin that computes maximum
 
 =head1 VERSION
 
-version 0.021
+version 0.023
 
 =head1 DESCRIPTION
 
@@ -72,7 +75,7 @@ This class implements an action for PDL::NDBin.
 	);
 
 Construct an instance for this action. Requires the number of bins $N as input.
-Optionally allows the type of the output piddle to be set (defaults to the type
+Optionally allows the type of the output ndarray to be set (defaults to the type
 of the variable this instance is associated with).
 
 =head2 process()
@@ -94,7 +97,7 @@ Edward Baudrez <ebaudrez@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Edward Baudrez.
+This software is copyright (c) 2022 by Edward Baudrez.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
