@@ -1,6 +1,6 @@
 package PDL::NDBin::Iterator;
 # ABSTRACT: Iterator object for PDL::NDBin
-$PDL::NDBin::Iterator::VERSION = '0.021';
+$PDL::NDBin::Iterator::VERSION = '0.023';
 use strict;
 use warnings;
 use Carp;
@@ -112,7 +112,7 @@ PDL::NDBin::Iterator - Iterator object for PDL::NDBin
 
 =head1 VERSION
 
-version 0.021
+version 0.023
 
 =head1 DESCRIPTION
 
@@ -140,16 +140,16 @@ in the second dimension. There must be at least one bin in every dimension.
 
 =item I<array>
 
-A reference to an array of piddles to operate on. The data values inside the
-piddles don't really matter, as far as the iterator object is concerned. The
-data inside the piddles will be made available to the actions in the order they
+A reference to an array of ndarrays to operate on. The data values inside the
+ndarrays don't really matter, as far as the iterator object is concerned. The
+data inside the ndarrays will be made available to the actions in the order they
 appear, one by one. There must be at least one element in this array.
 
 =item I<idx>
 
-A piddle containing the flattened bin numbers corresponding to the data values
-in the piddles in \@array. The length of this piddle must match the length of
-the piddles in \@array.
+A ndarray containing the flattened bin numbers corresponding to the data values
+in the ndarrays in \@array. The length of this ndarray must match the length of
+the ndarrays in \@array.
 
 =back
 
@@ -189,11 +189,11 @@ Return the number of variables.
 
 =head2 data()
 
-Return the piddle corresponding to the current variable.
+Return the ndarray corresponding to the current variable.
 
 =head2 idx()
 
-Return the piddle $idx passed to the constructor.
+Return the ndarray $idx passed to the constructor.
 
 =head2 var_active()
 
@@ -228,7 +228,7 @@ Another use is when empty bins needs to be skipped:
 
 	sub compute_maximum {
 		my $iter = shift;
-		# max() won't work with empty piddles
+		# max() won't work with empty ndarrays
 		return unless $iter->want->nelem;
 		my $values = $iter->selection;
 		return $values->max;
@@ -268,7 +268,7 @@ Edward Baudrez <ebaudrez@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Edward Baudrez.
+This software is copyright (c) 2022 by Edward Baudrez.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
